Method: OCI8.encoding

Defined in:
ext/oci8/encoding.c

.encodingObject

(new in ruby 1.9)

Returns Oracle client encoding.

String values passed to Oracle, such as SQL statements, bind values etc., are converted from their encoding to the Oracle client encoding.

If Encoding.default_internal is nil, string values got from Oracle are tagged by OCI8.encoding. If not nil, they are converted from OCI8.encoding to Encoding.default_internal by default.

If it is ‘ASCII-8BIT’, no encoding conversions are done.



157
158
159
160
# File 'ext/oci8/encoding.c', line 157

static VALUE oci8_get_encoding(VALUE klass)
{
    return rb_enc_from_encoding(oci8_encoding);
}