Class: OCIHandle
- Inherits:
-
Object
- Object
- OCIHandle
- Defined in:
- lib/oci8/ocihandle.rb
Overview
The abstract base class to implement common behavior of OCI handle data types and OCI descriptor data types.
The abstract base class to implement common behavior of OCI handle data types and OCI descriptor data types.